/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008 <talsi.elderme@gmail.com>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */


void directionalLight(	in vec3 lightDirection,
						in vec4 ambLight,
						in vec4 diffLight,
						in vec4 specLight,
						in vec3 vVec,
						in vec3 normal,
						in float shininess,
						inout vec4 ambient,
						inout vec4 diffuse,
						inout vec4 specular) {

	float diffuseCoeff;		
	float specularCoeff;

	ambient += ambLight;

	diffuseCoeff = max(0.0, dot(lightDirection, normal));
	diffuse += diffLight * diffuseCoeff;

	specularCoeff = dot(reflect(-lightDirection, normal), vVec);
	if(specularCoeff > 0.) {
		specularCoeff = pow(specularCoeff, shininess);
		specular += specLight * specularCoeff;
	}
}

void pointLight(in vec4 attenuationsAndDist,
				in vec3 lightDirection,
				in vec4 ambLight,
				in vec4 diffLight,
				in vec4 specLight,
				in vec3 vVec,
				in vec3 normal,
				in float shininess,
				inout vec4 ambient,
				inout vec4 diffuse,
				inout vec4 specular) {

	float attenuation;

	float diffuseCoeff;
	float specularCoeff;

	attenuation = 1.0/ (attenuationsAndDist[0] +
						attenuationsAndDist[1] * attenuationsAndDist[3] +
						attenuationsAndDist[2] *attenuationsAndDist[3]*attenuationsAndDist[3]);

	ambient += ambLight * attenuation;

	diffuseCoeff = max(0.0, dot(lightDirection, normal));
	diffuse += diffLight * diffuseCoeff * attenuation;

	specularCoeff = dot(reflect(-lightDirection, normal), vVec);
	if(specularCoeff > 0.) {
		specularCoeff = pow(specularCoeff, shininess);
		specular += specLight * specularCoeff * attenuation;
	}
}

void spotLight( in vec4 attenuationsAndDist,
				in vec3 lightPositionDir,
				in vec3 spotAxis,
				in vec4 ambLight,
				in vec4 diffLight,
				in vec4 specLight,
				in float exponent,
				in float cosCutoff,
				in vec3 vVec,
				in vec3 normal,
				in float shininess,
				inout vec4 ambient,
				inout vec4 diffuse,
				inout vec4 specular) {

	float attenuation;
	float spotAttenuation;

	float diffuseCoeff;
	float specularCoeff;

	spotAttenuation = dot(lightPositionDir, -spotAxis);
	if(spotAttenuation > cosCutoff) {
		spotAttenuation = pow(spotAttenuation, exponent);

		attenuation = 1.0/ (attenuationsAndDist[0] +
							attenuationsAndDist[1] * attenuationsAndDist[3] +
							attenuationsAndDist[2] *attenuationsAndDist[3]*attenuationsAndDist[3]);

		attenuation *=spotAttenuation;

		ambient += ambLight * attenuation;

		diffuseCoeff = max(0.0, dot(lightPositionDir, normal));
		diffuse += diffLight * diffuseCoeff * attenuation;

		specularCoeff = dot(reflect(-lightPositionDir, normal), vVec);
		if(specularCoeff > 0.) {
			specularCoeff = pow(specularCoeff, shininess);
			specular += specLight * specularCoeff * attenuation;
		}
	}
}

