/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _ENTITYWITHVALUE_H_
#define _ENTITYWITHVALUE_H_

#include "../newtEngine/entity.h"


class EntityWithValue: public Entity
{
public:
	EntityWithValue(Material *material, Mesh *mesh, int value, bool visible);
	virtual ~EntityWithValue();
	
	int getValue();
	void setValue(int value);
		
		
protected:
	int mValue;		
};


class EntityWithValueInLine: public EntityWithValue
{
public:
	EntityWithValueInLine(Material *material, Mesh *mesh, int value, int index, float interval, bool visible,
						  Vector3 axis);
	~EntityWithValueInLine();

	void applyTransformation();

		
protected:
	Vector3 position;
	int mIndex;			
		
};


class EntityWithValueInPlane: public EntityWithValue
{
public:
	EntityWithValueInPlane(Material *material, Mesh *mesh, int value, int index1, int index2, float interval, bool visible,
						   Vector3 axis1, Vector3 axis2);
	~EntityWithValueInPlane();

	void applyTransformation();

protected:
	Vector3 position;
	int mIndex1, mIndex2;
		
};


class EntityWithValueInTab: public EntityWithValue
{
public:
	EntityWithValueInTab(Material *material, Mesh *mesh, int value, int index1, int index2, int index3, float interval, bool visible,
						 Vector3 axis1, Vector3 axis2, Vector3 axis3);
	~EntityWithValueInTab();

	void applyTransformation();

protected:
	Vector3 position;
	int mIndex1, mIndex2, mIndex3;
		
};


class EntityWithValueInRing: public EntityWithValue
{
public:
	EntityWithValueInRing(Material *material, Mesh *mesh, int value, int index, float radius, float angleOffset, int nb, bool visible);	
	EntityWithValueInRing(Material *material, Mesh *mesh, int value, int index, float radius, float angleOffset, int nb, float theta, float phi, bool visible);
	~EntityWithValueInRing();

	void applyTransformation();

protected:
	float mRadius;
	float mAngle;
	float mPhi, mTheta;
	int mIndex;			
		
};






#endif // _ENTITYWITHVALUE_H_
