/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "textureLayer.h"

TextureLayer::TextureLayer(ImagePng *image, textureType type, GLenum wrapMode, bool mipmaps):
	mType(type), mBlendMode(GL_MODULATE), mWrapMode(wrapMode), mMagMode(GL_LINEAR), mScaleS(1.), mScaleT(1.) {
	glGenTextures (1, &id);	
	glBindTexture (GL_TEXTURE_2D, id);

	int internalFormat = image->getInternalFormat();
	GLuint width = image->getWidth();
	GLuint height = image->getHeight();
	GLenum format = image->getFormat();
	GLubyte *texels = image->getTexels();
		
	if(mipmaps) {
		mMinMode = GL_LINEAR_MIPMAP_LINEAR;
		gluBuild2DMipmaps(GL_TEXTURE_2D, internalFormat, width, height, format, GL_UNSIGNED_BYTE, texels);		
	} else {
		mMinMode = GL_LINEAR;		
		glTexImage2D(GL_TEXTURE_2D, 0, internalFormat, width, height, 0, format, GL_UNSIGNED_BYTE, texels);
	}	
}

TextureLayer::TextureLayer(int width, int height): 
	mType(OTHER_MAP), mBlendMode(GL_MODULATE), mWrapMode(GL_CLAMP_TO_EDGE), mMagMode(GL_LINEAR), mMinMode(GL_LINEAR_MIPMAP_LINEAR), mScaleS(1.), mScaleT(1.) {
	
	glGenTextures (1, &id);	
	
	glBindTexture(GL_TEXTURE_2D, id);
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA,  width, height, 0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
	
	glGenerateMipmapEXT(GL_TEXTURE_2D);		
}


TextureLayer::~TextureLayer() {	
	glDeleteTextures(1, &id);		
}


textureType TextureLayer::getType() {
	return mType;
}

GLenum TextureLayer::getId() {
	return id;
}


void TextureLayer::setBlendMode(GLenum blendMode) {
	mBlendMode = blendMode;
}

void TextureLayer::setScale(float scaleS, float scaleT) {
	mScaleS = scaleS;
	mScaleT = scaleT;
}


void TextureLayer::activate() {
	glMatrixMode (GL_TEXTURE);
	glLoadIdentity ();
	glScalef (mScaleS, mScaleT, 1.);
	glMatrixMode(GL_MODELVIEW);
	
	glEnable(GL_TEXTURE_2D);
    glBindTexture (GL_TEXTURE_2D, id);
	
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER, mMinMode);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER, mMagMode);

	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, mWrapMode);		
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, mWrapMode);	
	
    glTexEnvi(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,mBlendMode);	
}

void TextureLayer::release() {
	glDisable(GL_TEXTURE_2D);
}



















