/*
 * Newt-Puzzle
 * Copyright (C) Aude Jaulent 2008, 2009 <elderme@tuxfamily.org>
 * 
 * Newt-Puzzle is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * Newt-Puzzle is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "vector3.h"


Vector3::Vector3() : x(0.0), y(0.0), z(0.0) {	
}

Vector3::Vector3(float a, float b, float c) : x(a), y(b), z(c) {
}

float Vector3::getNorm() {
	return sqrt(x*x + y*y + z*z);
}

float Vector3::getNorm2(){
	return x*x + y*y + z*z;
}

float Vector3::normalize(){
	float norm =  sqrt(x*x + y*y + z*z);
	x = x /norm;
	y = y /norm;
	z = z /norm;
	return norm;
}

Vector3 Vector3::perpendicular(){
	Vector3 perp = Vector3(0.,z,-y);
	if(perp.getNorm2() < 0.00001) {
		perp = Vector3(-z, 0., x);	
	}
	return perp;
}

Vector3 Vector3::rotate(Vector3 axis, float theta) {
	Vector3 U = (axis*(*this))*axis;
	Vector3 V = (*this) - U;

	return U + cos(theta)*V + (sin(theta)*axis^V);	
}


Vector3 operator+(const Vector3& vect1,const Vector3& vect2) {
	return Vector3(vect1.x + vect2.x, vect1.y + vect2.y, vect1.z + vect2.z);
}

Vector3 operator-(const Vector3& vect1,const Vector3& vect2) {
	return Vector3(vect1.x - vect2.x, vect1.y - vect2.y, vect1.z - vect2.z);
}

Vector3 operator*(const Vector3& vect, const float scalar) {
	return Vector3(scalar*vect.x,scalar*vect.y,scalar*vect.z);
}

Vector3 operator*(const float scalar, const Vector3& vect) {
	return Vector3(scalar*vect.x,scalar*vect.y,scalar*vect.z);
}


float operator*(const Vector3& vect1,const Vector3& vect2) {
	return vect1.x * vect2.x + vect1.y * vect2.y + vect1.z * vect2.z;
}

Vector3 operator^(const Vector3& vect1,const Vector3& vect2) {
	return Vector3(	
				vect1.y * vect2.z - vect1.z * vect2.y,
				vect1.z * vect2.x - vect1.x * vect2.z,
				vect1.x * vect2.y - vect1.y * vect2.x);
}	
	
	
	
	
	
